import numpy

class SimpleBeatDetection:
		"""
		Simple beat detection algorithm from
		http://archive.gamedev.net/archive/reference/programming/features/beatdetection/index.html
		"""
		def __init__(self, history = 43):
				self.local_energy = numpy.zeros(history) # a simple ring buffer
				self.local_energy_index = 0 # the index of the oldest element

		def detect_beat(self, signal):

				samples = signal.astype(numpy.int) # make room for squares
				# optimized sum of squares, i.e faster version of (samples**2).sum()
				instant_energy = abs(numpy.dot(samples, samples) / float(0xffffffff)) # normalize

				local_energy_average = self.local_energy.mean()
				local_energy_variance = self.local_energy.var()
														#-0.0025714
				beat_sensibility = (-20 * local_energy_variance) +2#+ 1.15142857
				beat = instant_energy > beat_sensibility * local_energy_average
				
				#print(" 1" if beat else "0 ", instant_energy/ beat_sensibility / local_energy_average, local_energy_variance)

				self.local_energy[self.local_energy_index] = instant_energy
				self.local_energy_index -= 1
				if self.local_energy_index < 0:
						self.local_energy_index = len(self.local_energy) - 1

				return beat